"""Django settings used during a container build; used
mainly to support ``collectstatic``.
"""
from unimatrix.ext.django.settings.const import *

DEBUG = False

INSTALLED_APPS = [
    'django.contrib.staticfiles'
]

SECRET_KEY = ('0' * 32)

STATIC_URL = '/assets/'



import importlib
import os
import sys
import inspect

from django.conf import settings
from django.conf.urls.static import static
from django.urls import path
from django.urls import include


urlpatterns = []

if os.environ.get('UNIMATRIX_URLS_MODULE'):
    self = sys.modules[__name__]
    urls = importlib.import_module(os.environ['UNIMATRIX_URLS_MODULE'])
    if not hasattr(urls, 'urlpatterns'):
        raise ValueError(
            "%s must declare urlpatterns" % os.environ['UNIMATRIX_SETTINGS_MODULE'])

    if getattr(settings, 'STATIC_SERVE', False):
        urlpatterns += static(settings.STATIC_URL,
            document_root=settings.STATIC_ROOT)


    # Provide a configuration to enable or disable the
    # Django Rest Framework API browser.
    if getattr(settings, 'API_BROWSER_ENABLED', False)\
    and 'rest_framework' in settings.INSTALLED_APPS:
        urlpatterns += [
            path(getattr(settings, 'API_BROWSER_PATH', 'browse/'),
                include('rest_framework.urls'))
        ]



    urlpatterns.extend(urls.urlpatterns)



import os

from django.db.utils import DEFAULT_DB_ALIAS
import yaml

from unimatrix.const import DB_CONNECTION
from unimatrix.const import PKIDIR
from unimatrix.const import TLS_ENFORCE
from unimatrix.lib.datastructures import DTO


DATABASE_ENGINES = {
    # Use verify-ca instead of verify-full since we can not guarantee
    # that the server hostname matches that of the CA certificate.
    'postgresql': {
        'port': 5432,
        'backend': 'django.db.backends.postgresql',
        'tls.server': lambda ca: {
            'sslmode': 'verify-ca',
            'sslrootcert': ca
        },
    },
    'postgis': {
        'port': 5432,
        'backend': 'django.contrib.gis.db.backends.postgis',
        'tls.server': lambda ca: {
            'sslmode': 'verify-ca',
            'sslrootcert': ca
        },
    },
    'memory': {
        'port': None,
        'name': ':memory:'
    }
    #'mysql': {
    #    'port': 3306,
    #    'backend': 'django.db.backends.mysql'
    #}
}


def load_databases(dirname):
    """Loads the databases from the common database connection
    format.
    """
    connections = {}
    defaults = {
        DEFAULT_DB_ALIAS: {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
      }
    }
    if not os.path.exists(dirname):
        return defaults
    for alias in os.listdir(dirname):
        fp = os.path.join(dirname, alias)
        if str.startswith(alias, '.') or os.path.isdir(fp):
            continue
        with open(fp) as f:
            dto = DTO.fromdict(yaml.safe_load(f.read()) or {})
        if not dto:
            continue
        x509_ca = os.path.join(PKIDIR, 'server', f'rdbms.{alias}.crt')
        x509_crt = os.path.join(PKIDIR, 'client', f'rdbms.{alias}.crt')
        x509_key = os.path.join(PKIDIR, 'client', f'rdbms.{alias}.key')
        if alias == DB_CONNECTION:
            alias = DEFAULT_DB_ALIAS
        if dto.engine not in DATABASE_ENGINES:
            raise ValueError(f"Unsupported engine: {dto.engine}")
        connection = {
            'ENGINE': DATABASE_ENGINES[dto.engine]['backend'],
            'HOST': dto.get('host'),
            'PORT': dto.get('port') or DATABASE_ENGINES[dto.engine]['port'],
            'NAME': dto.get('name')\
                or DATABASE_ENGINES[dto.engine].get('name'),
            'USER': dto.get('user'),
            'PASSWORD': dto.get('password'),
            'CONN_MAX_AGE': dto.get('max_age') or None,
            'OPTIONS': {}
        }

        #if TLS_ENFORCE:
        #    for pem in (x509_ca, x509_crt, x509_key)[:1]:
        #        if not os.path.exists(pem):
        #            raise Exception(f"Missing certificate: {pem}")
        if os.path.exists(x509_ca):
            connection['OPTIONS'].update(
                DATABASE_ENGINES[dto.engine]['tls.server'](x509_ca))

        if os.path.exists(x509_crt)\
        and os.path.exists(x509_key):
            raise NotImplementedError

        connections[alias] = connection

    return connections or defaults



"""Constants used in the Unimatrix One/CM platforms,
for deployments for Django.
"""
import os


DEFAULT_LOG_LEVEL = 'ERROR'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
        'null': {
            'class': 'logging.NullHandler',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'django.db.backends': {
            'handlers': ['null'],
            'propagate': False,
        },
        'proton': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'aorta': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'drone': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
    },
}

SESSIONS_PERSISTENT = os.getenv('SESSIONS_PERSISTENT') == '1'

STATICFILES_DIRS = ['dist/assets']

STATIC_ROOT = 'static'
